import { error } from "@sveltejs/kit";
import { saveScore } from "$lib";
import type { RequestHandler } from "./$types";
import type { CheckData } from "$lib/types";

export const POST: RequestHandler = async ({ request, platform }) => {
	const kv = platform?.env?.TCL_GUESSR_KV;
	if (!kv) error(500, "could not connect to kv");

	const data: CheckData = await request.json();

	const solution = await saveScore(
		data.gameId,
		data.index,
		(solution) => calcDistance(data.latlng, solution),
		(distance) => calculatePoints(distance),
		kv,
	);

	if (solution) {
		return Response.json(solution);
	} else {
		return new Response(null, { status: 404 });
	}
};

function calculatePoints(distance: number): number {
	const lenientDistance = Math.max(0, distance - 20);
	const score = 5000 * Math.exp(-lenientDistance / 750);

	const multiplier = 1; /*
    if (lines) multiplier *= 0.5;
    if (labels) multiplier *= 0.5; */

	return score * multiplier;
}

// Mean Earth Radius, as recommended for use by
// the International Union of Geodesy and Geophysics,
// see https://rosettacode.org/wiki/Haversine_formula
const R = 6371000;

// https://github.com/Leaflet/Leaflet/blob/142f94a9ba5757f7e7180ffa6cbed2b3a9bc73c9/src/geo/crs/CRS.Earth.js#L23
function calcDistance(latlng1: [number, number], latlng2: [number, number]): number {
	const rad = Math.PI / 180,
		lat1 = latlng1[0] * rad,
		lat2 = latlng2[0] * rad,
		sinDLat = Math.sin(((latlng2[0] - latlng1[0]) * rad) / 2),
		sinDLon = Math.sin(((latlng2[1] - latlng1[1]) * rad) / 2),
		a = sinDLat * sinDLat + Math.cos(lat1) * Math.cos(lat2) * sinDLon * sinDLon,
		c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
	return R * c;
}
