import { error } from "@sveltejs/kit";
import type { RequestHandler } from "./$types";
import { createGame, getMergedStops, getMetro, getTram, parseOptions, sample } from "$lib";

export const GET: RequestHandler = async ({ fetch, url, platform }) => {
	const kv = platform?.env?.TCL_GUESSR_KV;
	if (!kv) error(500, "could not connect to kv");

	const options = parseOptions(url);
	const lineColors: [GeoJSON.Feature, string][] = [];

	if (options.metro) lineColors.push(...(await getMetro(fetch)));
	if (options.tram) lineColors.push(...(await getTram(fetch)));

	const lineCodes = new Set<string>(lineColors.map(([f]) => f.properties!.code_ligne));
	const crossingStops = await getMergedStops(fetch, lineCodes);

	const randomStops = sample(crossingStops, 5);
	if (!randomStops) {
		error(400, "could not select random stop");
	}

	const gameData = await createGame(options.mode, randomStops, kv);

	return Response.json(gameData);
};
